# Candidate Manager

The **Candidate Manager** is a custom Drupal module that provides entity management for **Candidates** and **Companies**, complete with CRUD operations, profile views, admin dashboard, filtering, and REST API support.

---

## 🚀 Features

- Custom entities: **Candidate** and **Company**
- Full CRUD interfaces via admin UI
- Public profile pages for both entities
- REST API support (GET, POST, PATCH, DELETE)
- Filtering on candidate list
- Admin dashboard with quick links
- Bootstrap-compatible templates
- Drupal coding standards and best practices
- Custom taxonomy vocabularies: Skills and Category
- Settings form and configurable display options
- Help page integration

---

## 📁 File Structure Overview

```plaintext
modules/
└── candidate_manager/
    ├── config/
    │   └── install/
    │       └── rest.resource.candidate_rest_resource.yml
    ├── css/
    │   ├── admin-dashboard.css
    │   ├── candidate-profile.css
    │   ├── help.css
    ├── src/
    │   ├── Controller/
    │   │   ├── AdminController.php
    │   │   ├── CandidateController.php
    │   │   ├── CompanyController.php
    │   │   └── HelpController.php
    │   ├── Entity/
    │   │   ├── Candidate.php
    │   │   └── Company.php
    │   ├── Form/
    │   │   ├── CandidateListFilterForm.php
    │   │   └── CandidateManagerSettingsForm.php
    │   ├── Plugin/
    │   │   └── rest/
    │   │       └── resource/
    │   │           └── CandidateRestResource.php
    │   ├── CandidateListBuilder.php
    │   └── CompanyListBuilder.php
    ├── templates/
    │   ├── candidate-profile.html.twig
    │   ├── company-profile.html.twig
    │   └── page--candidate--view.html.twig
    ├── candidate_manager.info.yml
    ├── candidate_manager.install
    ├── candidate_manager.links.menu.yml
    ├── candidate_manager.libraries.yml
    ├── candidate_manager.permissions.yml
    ├── candidate_manager.module
    ├── candidate_manager.routing.yml
    └── README.md
```

---

## 🔗 Accessible Routes

### 🧑 Candidates

| Feature             | Path                                         |
|---------------------|----------------------------------------------|
| Candidate List      | `/admin/candidates`                         |
| Add Candidate       | `/admin/candidates/add`                     |
| Edit Candidate      | `/admin/candidates/{candidate}/edit`        |
| Delete Candidate    | `/admin/candidates/{candidate}/delete`      |
| View Candidate      | `/candidate/{candidate}`                    |

---

### 🏢 Companies

| Feature             | Path                                         |
|---------------------|----------------------------------------------|
| Company List        | `/admin/companies`                          |
| Add Company         | `/admin/companies/add`                      |
| Edit Company        | `/admin/companies/{company}/edit`           |
| Delete Company      | `/admin/companies/{company}/delete`         |
| View Company        | `/company/{company}`                        |

---

### 🧭 Admin Dashboard

| Feature                      | Path                                |
|------------------------------|--------------------------------------|
| Candidate Manager Dashboard  | `/admin/config/candidate-manager`   |

Provides quick access to:

- ✅ Manage Candidates
- ✅ Manage Companies
- ✅ Candidate Manager Settings

---

## 🧩 Settings and Help

- **Settings Page**: `/admin/config/candidate-manager/settings`
- **Help Page**: `/admin/help/candidate_manager`

Settings allow switching between display modes and help includes module overview and guidance.

---

## 📡 REST API

REST API is available for the `Candidate` entity.

### Endpoints

| Method | Endpoint              | Description                        |
|--------|-----------------------|------------------------------------|
| GET    | `/api/candidates/{id}` | Retrieve a candidate by ID        |
| POST   | `/api/candidates`      | Create a new candidate             |
| PATCH  | `/api/candidates/{id}` | Update an existing candidate       |
| DELETE | `/api/candidates/{id}` | Delete a candidate                 |

### Authentication

- **Basic Auth** is required.
- The user must have the `administer candidate entities` permission.

### Example: Create a Candidate via cURL

```bash
curl -X POST https://projectx.lndo.site/api/candidates \
  -H "Content-Type: application/json" \
  -H "Accept: application/json" \
  --user admin:yourpassword \
  -d '{
    "full_name": "Jane Doe",
    "email": "jane.doe@example.com"
  }'
```

> ✅ Make sure the `REST` and `serialization` modules are enabled and the REST resource is configured at `/admin/config/services/rest`.

---

## 🧾 Settings Page

The settings form is available at:

- `/admin/config/candidate-manager/settings`

Implemented via `CandidateManagerSettingsForm`, it supports configuration like display mode toggles and advanced layout rendering.

---

## 🧠 Taxonomies

The module creates two vocabularies:

- **Skills** (e.g., PHP, JavaScript, React, Drupal)
- **Category** (e.g., Developer, Designer, Tester, Manager)

These are installed and populated automatically.

---

## 🧩 Templates

- `candidate-profile.html.twig` — Renders individual candidate details.
- `company-profile.html.twig` — Renders individual company details.
- `page--candidate--view.html.twig` — Alternative page template for candidate view.

These templates use Bootstrap-based classes (`container`, `card`, `mt-4`, etc.) for clean UI integration.

---

## 🧪 Filters

Candidate list includes filter functionality implemented using `CandidateListFilterForm`. Filters include:

- Skills
- Category

Integrated directly into the CandidateListBuilder output.

---

## 🛠️ Hooks and Theming

- Implements `hook_theme()` for custom templates
- Uses `hook_help()` to provide contextual help
- Registers theme suggestions conditionally using `hook_theme_suggestions_page_alter()`

---

## ✅ Future Enhancements

- Dynamic breadcrumb support
- Views integration for listing/filtering
- Relationships between Candidates and Companies
- JSON:API support for broader integrations

---

## 📦 Requirements

- Drupal 10+ or 11
- Admin access to configure entities and permissions

---

## 📥 Installation

1. Place this module under `web/modules/custom/candidate_manager/`
2. Enable via Admin UI or Drush:

```bash
drush en candidate_manager
```

3. Configure permissions and start managing candidates & companies!

---

## 👨 Maintainer

This module is maintained by **Dipanjan Kundu**.

This module includes the Parsedown library (https://github.com/erusev/parsedown),
which is distributed under the MIT license. A copy of the license is included in this repository.
