<?php

namespace Drupal\candidate_manager;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Url;
use Drupal\candidate_manager\Entity\Candidate;
use Drupal\candidate_manager\Form\CandidateListFilterForm;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a class to build a listing of Candidate entities.
 */
class CandidateListBuilder extends EntityListBuilder {

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $currentRequest;

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, $entity_type) {
    $instance = parent::createInstance($container, $entity_type);
    $instance->currentRequest = $container->get('request_stack')->getCurrentRequest();
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader(): array {
    $header['id'] = $this->t('ID');
    $header['name'] = $this->t('Name');
    $header['email'] = $this->t('Email');
    $header['phone'] = $this->t('Phone');
    $header['status'] = $this->t('Status');
    $header['linkedin_url'] = $this->t('LinkedIn');
    $header['available_from'] = $this->t('Available');
    $header['location'] = $this->t('Location');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity): array {
    /** @var \Drupal\candidate_manager\Entity\Candidate $entity */
    $row['id'] = $entity->id();
    $row['name'] = $entity->toLink();
    $row['email'] = $entity->get('email')->value;
    $row['phone'] = $entity->get('phone')->value;
    $row['status'] = $entity->get('status')->value;
    $row['linkedin_url'] = $entity->get('linkedin_url')->uri ?? '';
    $row['available_from'] = $entity->get('location')->value ? $this->t('Yes') : $this->t('No');
    $row['location'] = $entity->get('location')->value;
    return $row + parent::buildRow($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function render(): array {
    $build = [];

    // Add the filter form.
    $build['filter_form'] = \Drupal::formBuilder()->getForm(CandidateListFilterForm::class);

    // Now the table.
    $build += parent::render();

    $build['add'] = [
      '#type' => 'link',
      '#title' => $this->t('Add Candidate'),
      '#url' => Url::fromRoute('entity.candidate.add_form'),
      '#attributes' => ['class' => ['button', 'button--primary']],
      '#weight' => -10,
    ];

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function load() {
    $query = $this->getStorage()->getQuery()->accessCheck(TRUE);
    $query->sort($this->entityType->getKey('id'));

    // Get filter values from the request.
    $skill_tid = $this->currentRequest->query->get('skill');
    $category_tid = $this->currentRequest->query->get('category');

    if ($skill_tid) {
      $query->condition('skills.target_id', $skill_tid);
    }
    if ($category_tid) {
      $query->condition('category.target_id', $category_tid);
    }

    $ids = $query->execute();
    return $this->getStorage()->loadMultiple($ids);
  }

}
