<?php

namespace Drupal\candidate_manager\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\user\EntityOwnerTrait;
use Drupal\Core\Entity\EntityChangedTrait;

/**
 * Defines the Candidate entity.
 *
 * @ContentEntityType(
 *   id = "candidate",
 *   label = @Translation("Candidate"),
 *   base_table = "candidate",
 *   entity_keys = {
 *     "id" = "id",
 *     "uuid" = "uuid",
 *     "label" = "full_name",
 *     "uid" = "user_id"
 *   },
 *   handlers = {
 *     "list_builder" = "Drupal\candidate_manager\CandidateListBuilder",
 *     "storage" = "Drupal\Core\Entity\Sql\SqlContentEntityStorage",
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "access" = "Drupal\Core\Entity\EntityAccessControlHandler",
 *     "form" = {
 *       "default" = "Drupal\Core\Entity\ContentEntityForm",
 *       "add" = "Drupal\Core\Entity\ContentEntityForm",
 *       "edit" = "Drupal\Core\Entity\ContentEntityForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm"
 *     },
 *     "route_provider" = {
 *       "default" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *       "field_ui" = "Drupal\Core\Entity\Routing\DefaultHtmlRouteProvider"
 *     }
 *   },
 *   admin_permission = "administer candidate entities",
 *   links = {
 *     "canonical" = "/candidate/{candidate}",
 *     "add-form" = "/admin/candidates/add",
 *     "edit-form" = "/admin/candidates/{candidate}/edit",
 *     "delete-form" = "/admin/candidates/{candidate}/delete",
 *     "collection" = "/admin/candidates"
 *   },
 *   has_list = TRUE
 * )
 */
class Candidate extends ContentEntityBase {
  use EntityOwnerTrait;
  use EntityChangedTrait;

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type): array {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['full_name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Full Name'))
      ->setRequired(TRUE)
      ->setSettings(['max_length' => 255])
      ->setDisplayOptions('form', ['type' => 'string_textfield', 'weight' => 0])
      ->setDisplayOptions('view', ['label' => 'above', 'type' => 'string', 'weight' => 0])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['email'] = BaseFieldDefinition::create('email')
      ->setLabel(t('Email'))
      ->setRequired(TRUE)
      ->setDisplayOptions('form', ['type' => 'email_default', 'weight' => 1])
      ->setDisplayOptions('view', ['label' => 'above', 'type' => 'email_mailto', 'weight' => 1])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['phone'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Phone'))
      ->setSettings(['max_length' => 20])
      ->setDisplayOptions('form', ['type' => 'string_textfield', 'weight' => 2])
      ->setDisplayOptions('view', ['label' => 'above', 'type' => 'string', 'weight' => 2])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['resume'] = BaseFieldDefinition::create('file')
      ->setLabel(t('Resume'))
      ->setSettings([
        'uri_scheme' => 'public',
        'file_extensions' => 'pdf doc docx',
      ])
      ->setDisplayOptions('form', ['type' => 'file_generic', 'weight' => 3])
      ->setDisplayOptions('view', ['label' => 'above', 'type' => 'file_default', 'weight' => 3])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['notes'] = BaseFieldDefinition::create('text_long')
      ->setLabel(t('Notes'))
      ->setDisplayOptions('form', ['type' => 'text_textarea', 'weight' => 4])
      ->setDisplayOptions('view', ['label' => 'above', 'type' => 'text_default', 'weight' => 4])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['skills'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Skills'))
      ->setCardinality(BaseFieldDefinition::CARDINALITY_UNLIMITED)
      ->setSetting('target_type', 'taxonomy_term')
      ->setSetting('handler_settings', ['target_bundles' => ['skills' => 'skills']])
      ->setDisplayOptions('form', ['type' => 'entity_reference_autocomplete', 'weight' => 5])
      ->setDisplayOptions('view', ['label' => 'above', 'type' => 'entity_reference_label', 'weight' => 5])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['category'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Category'))
      ->setSetting('target_type', 'taxonomy_term')
      ->setSetting('handler_settings', ['target_bundles' => ['category' => 'category']])
      ->setDisplayOptions('form', ['type' => 'options_select', 'weight' => 6])
      ->setDisplayOptions('view', ['label' => 'above', 'type' => 'entity_reference_label', 'weight' => 6])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['company'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Company'))
      ->setSetting('target_type', 'company')
      ->setDisplayOptions('form', ['type' => 'entity_reference_autocomplete', 'weight' => 7])
      ->setDisplayOptions('view', ['label' => 'above', 'type' => 'entity_reference_label', 'weight' => 7])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['user_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Author'))
      ->setSetting('target_type', 'user')
      ->setDefaultValueCallback('Drupal\candidate_manager\Entity\Candidate::getCurrentUserId')
      ->setDisplayOptions('form', ['type' => 'entity_reference_autocomplete', 'weight' => 8])
      ->setDisplayOptions('view', ['label' => 'above', 'type' => 'author', 'weight' => 8])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['status'] = BaseFieldDefinition::create('list_string')
      ->setLabel(t('Status'))
      ->setSettings(['allowed_values' => [
        'applied' => 'Applied',
        'shortlisted' => 'Shortlisted',
        'interviewing' => 'Interviewing',
        'hired' => 'Hired',
        'rejected' => 'Rejected',
      ]])
      ->setDefaultValue('applied')
      ->setDisplayOptions('form', ['type' => 'options_select', 'weight' => 9])
      ->setDisplayOptions('view', ['label' => 'above', 'type' => 'list_default', 'weight' => 9])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['linkedin_url'] = BaseFieldDefinition::create('link')
      ->setLabel(t('LinkedIn Profile'))
      ->setDisplayOptions('form', ['type' => 'link_default', 'weight' => 10])
      ->setDisplayOptions('view', ['label' => 'above', 'type' => 'link_default', 'weight' => 10])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['available_from'] = BaseFieldDefinition::create('datetime')
      ->setLabel(t('Available From'))
      ->setSetting('datetime_type', 'date')
      ->setDisplayOptions('form', ['type' => 'datetime_default', 'weight' => 11])
      ->setDisplayOptions('view', ['label' => 'above', 'type' => 'datetime_default', 'weight' => 11])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['location'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Location'))
      ->setSettings(['max_length' => 255])
      ->setDisplayOptions('form', ['type' => 'string_textfield', 'weight' => 13])
      ->setDisplayOptions('view', ['label' => 'above', 'type' => 'string', 'weight' => 13])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    return $fields;
  }

  /**
   * Returns the ID of the current user.
   *
   * @return int
   *   The current user's ID.
   */
  public static function getCurrentUserId() {
    return \Drupal::currentUser()->id();
  }
}
