<?php

namespace Drupal\candidate_manager\Plugin\rest\resource;

use Drupal\candidate_manager\Entity\Candidate;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Psr\Log\LoggerInterface;

/**
 * Provides a REST resource for Candidate entity.
 *
 * @RestResource(
 *   id = "candidate_rest_resource",
 *   label = @Translation("Candidate REST Resource"),
 *   uri_paths = {
 *     "canonical" = "/api/candidates/{id}",
 *     "create" = "/api/candidates"
 *   }
 * )
 */
class CandidateRestResource extends ResourceBase {

  protected EntityTypeManagerInterface $entityTypeManager;
  protected AccountProxyInterface $currentUser;

  public function __construct(
    array $configuration,
          $plugin_id,
          $plugin_definition,
    array $serializer_formats,
    LoggerInterface $logger,
    AccountProxyInterface $current_user,
    EntityTypeManagerInterface $entity_type_manager
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->currentUser = $current_user;
    $this->entityTypeManager = $entity_type_manager;
  }

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('candidate_rest_resource'),
      $container->get('current_user'),
      $container->get('entity_type.manager')
    );
  }

  public function get($id = NULL): ResourceResponse {
    $candidate = Candidate::load($id);
    if (!$candidate) {
      throw new NotFoundHttpException("Candidate with ID $id not found.");
    }
    return new ResourceResponse($candidate, 200);
  }

  public function post(Request $request): ResourceResponse {
    if (!$this->currentUser->hasPermission('administer candidate entities')) {
      throw new AccessDeniedHttpException();
    }

    $data = json_decode($request->getContent(), TRUE);
    if (!$data || empty($data['full_name']) || empty($data['email'])) {
      throw new BadRequestHttpException('Missing required fields.');
    }

    $candidate = Candidate::create($data);
    $candidate->save();

    return new ResourceResponse([
      'message' => 'Candidate created successfully.',
      'id' => $candidate->id(),
    ], 201);
  }

  public function patch(Request $request, $id = NULL): ResourceResponse {
    if (!$this->currentUser->hasPermission('administer candidate entities')) {
      throw new AccessDeniedHttpException();
    }

    $candidate = Candidate::load($id);
    if (!$candidate) {
      throw new NotFoundHttpException("Candidate with ID $id not found.");
    }

    $data = json_decode($request->getContent(), TRUE);
    foreach ($data as $field => $value) {
      if ($candidate->hasField($field)) {
        $candidate->set($field, $value);
      }
    }
    $candidate->save();

    return new ResourceResponse($candidate, 200);
  }

  public function delete($id = NULL): ResourceResponse {
    if (!$this->currentUser->hasPermission('administer candidate entities')) {
      throw new AccessDeniedHttpException();
    }

    $candidate = Candidate::load($id);
    if (!$candidate) {
      throw new NotFoundHttpException("Candidate with ID $id not found.");
    }

    $candidate->delete();

    return new ResourceResponse([
      'message' => "Candidate $id deleted successfully."
    ], 200);
  }

}
